/*----------------------------------------------/
 
 * Project X3 FX Shader File
 
 * File Author: Wilham Anggowo Putro ( AeroFX )

 * File Version: 8.2.164.01 ( December 2015 )

 * Copyright (c) 2014 - 2015 Wilham Anggowo Putro

 *----------------------------------------------/

 * ENBSeries Main FX file

 * Visit http://enbdev.com for Updates

 * Copyright (c) 2007 - 2014 Boris Vorontsov

 *----------------------------------------------*/
#define BLOOM_MIX_TYPE		0		// Bloom Mixing Method (1-3)

	//HAVE FUN ;)

//COLORS
float BBrightDay		= 1.0;
float BSaturationDay		= 1.0;
float BBlendDay			= 1.5;
float BBlend2Day		= 0.0001;

float BBrightNight		= 1.0;
float BSaturationNight		= 1.0;
float BBlendNight		= 1.5;
float BBlend2Night		= 0.0001;

//[POST PROCESSING 01]
float PPSaturationDay		=  1.1;
float PPBrightnessDay		=  1.1;
float PPContrastDay			=  1.1;
float PPCurveDay			=  1.0;
float PPFinalContrastDay	=  0.2;
float PPExposureDay			=  0.01;

float PPSaturationNight		=  1.1;
float PPBrightnessNight		=  1.1;
float PPContrastNight		=  1.1;
float PPCurveNight			=  1.0;
float PPFinalContrastNight	=  0.2;
float PPExposureNight		=  0.01;

/*----------------------------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------------------------*/
float4	tempF1; 		//0,1,2,3
float4	tempF2; 		//5,6,7,8
float4	tempF3; 		//9,0
float4	Timer;			//x=generic timer in range 0..1, period of 16777216 ms (4.6 hours), w=frame time elapsed (in seconds)
float4	ScreenSize;		//x=Width, y=1/Width, z=ScreenScaleY, w=1/ScreenScaleY
float	ENightDayFactor;	//changes in range 0..1, 0 means that night time, 1 - day time
float	EInteriorFactor;	//changes 0 or 1. 0 means that exterior, 1 - interior
float	EAdaptiveQualityFactor;	//changes in range 0..1, 0 means full quality, 1 lowest dynamic quality (0.33, 0.66 are limits for quality levels)
float	EBloomAmount;		//enb version of bloom applied, ignored if original post processing used
float	FieldOfView;		//fov in degrees

/*----------------------------------------------------------------------------------------------------------------*/

/*----------------/
/---- TEXTURE ----/
/----------------*/
 
texture2D texs0;	//color
texture2D texs3;	//bloom enb
texture2D texs4;	//adaptation enb
texture2D texs7;	//palette enb
/*-------------------/
/----- SAMPLERS -----/
/-------------------*/

sampler2D _s0 = sampler_state
{
	Texture   = <texs0>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D _s3 = sampler_state
{
	Texture   = <texs3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D _s4 = sampler_state
{
	Texture   = <texs4>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D _s7 = sampler_state
{
	Texture   = <texs7>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};


/*----------------------------/
/----- SHADER STRUCTURE ------/
/----------------------------*/
  
  
struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord0 : TEXCOORD0;
};
struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord0 : TEXCOORD0;
};



/*-----------------------------/
/------- VERTEX SHADER  -------/
/-----------------------------*/

VS_OUTPUT_POST VS_Quad(VS_INPUT_POST IN)
{
	VS_OUTPUT_POST OUT;

	OUT.vpos=float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);

	OUT.txcoord0.xy=IN.txcoord0.xy;

	return OUT;
}

/*-----------------------------/
/------- PIXEL SHADERS --------/
/-----------------------------*/

float4 PS_GTASA(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	//Outputs
	
	//Here the real variables
	float4 color = tex2D(_s0, IN.txcoord0.xy);
	float4 bloom = tex2D(_s3, IN.txcoord0.xy);
	float4 adapt = tex2D(_s4, 0.5);
	float2  pixelsize 	= float2(ScreenSize.y,ScreenSize.y*ScreenSize.z);
	
//BLOOM
		float BBright = lerp( BBrightNight, BBrightDay, ENightDayFactor);
		float BSaturation = lerp( BSaturationNight, BSaturationDay, ENightDayFactor);
		float BBlend = lerp( BBlendNight, BBlendDay, ENightDayFactor);
		float BBlend2 = lerp( BBlend2Night, BBlend2Day, ENightDayFactor);
			
	float3 cgray2=dot(bloom.xyz, BBright);
	float3 poweredcolor2=pow(bloom.xyz, BSaturation);
	float newgray2=dot(poweredcolor2.xyz, BBlend);
	bloom.xyz=poweredcolor2.xyz*cgray2/(newgray2+BBlend2);		

	#if ( BLOOM_MIX_TYPE == 0 )
	color.xyz+=bloom.xyz * EBloomAmount;
	#endif
	#if ( BLOOM_MIX_TYPE == 1 )
	color.xyz = lerp(color.xyz, bloom.xyz, 0.0001);
	#endif
	#if ( BLOOM_MIX_TYPE == 2 )
	float3 toBlend = saturate(bloom.xyz * EBloomAmount);
	color.xyz = 1-(1-color.xyz)*(1-toBlend.xyz);
	#endif
	#if ( BLOOM_MIX_TYPE == 3 )
	color.xyz *= ( 1 - saturate( bloom.xyz * EBloomAmount ));
	color.xyz += bloom.xyz * EBloomAmount;
	#endif
	#if (BLOOM_MIX_TYPE == 4)
	float3 toBlend = saturate(bloom.xyz * EBloomAmount);
	color.xyz = 1-(1-color.xyz)*(1-toBlend.xyz);
	color.xyz = lerp(color.xyz, bloom.xyz, toBlend);
	#endif
	
//POST PROCESSING
	float PPSaturation = lerp( PPSaturationNight, PPSaturationDay, ENightDayFactor);
	float PPBrightness = lerp( PPBrightnessNight, PPBrightnessDay, ENightDayFactor);
	float PPContrast = lerp( PPContrastNight, PPContrastDay, ENightDayFactor);
	float PPCurve = lerp( PPCurveNight, PPCurveDay, ENightDayFactor);
	float PPFinalContrast = lerp( PPFinalContrastNight, PPFinalContrastDay, ENightDayFactor);
	float PPExposure = lerp( PPExposureNight, PPExposureDay, ENightDayFactor);

		float3  xncol;
		float3  scl;
		float4 	oldcolor;	
		oldcolor 				= color;
		xncol					= normalize(color.xyz);
		scl					= color.xyz/xncol.xyz;
		xncol.xyz				= pow(xncol.xyz, PPSaturation);
		color.xyz				= scl*xncol.xyz;
		color.xyz				= pow(color, 1.0 / PPBrightness);
		color.r 				= 1.0/(1.0+color.r);
		color.g 				= 1.0/(1.0+color.g);
		color.b 				= 1.0/(1.0+color.b);
		color.xyz				= pow(color, PPContrast) / (pow(color, PPContrast) + pow((1.0 - color), PPContrast));
		color.r 				= (1.0/color.r)-1.0;
		color.g 				= (1.0/color.g)-1.0;
		color.b 				= (1.0/color.b)-1.0;
		color.xyz 				= (color.xyz * (1.0 + color.xyz/1.0))/(color.xyz + PPCurve);
		color.xyz 				= lerp(color.xyz, 0.5 * (1 + sin((color.xyz - 0.5)*3.1415926)), PPFinalContrast);
		color.rgb 				*= pow(2.0f, PPExposure);
		color 					= lerp(oldcolor, color, 1.0);

// TONEMAPPING
	color.w = 1.0;
	
	
	return color;
}

/*----------------------/
/----- TECHNIQUE -------/
/----------------------*/

technique Shader_GTASA <string UIName="Project X3";>
{
	pass p0
	{
		VertexShader = compile vs_3_0 VS_Quad();
		PixelShader  = compile ps_3_0 PS_GTASA();

		ColorWriteEnable=ALPHA|RED|GREEN|BLUE;
		ZEnable=FALSE;
		ZWriteEnable=FALSE;
		CullMode=NONE;
		AlphaTestEnable=FALSE;
		AlphaBlendEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
}
